function addTheaterObject()
{
	var o = Obj3D.newEmpty();
	o.add(Obj3D.load("theater")).transform = Matrix.newScale(0.3);
	GLDisplay.world3d.add(o);
	return o;
}

function createGlassMaterial()
{
	var mat = Material.new();
	mat.lighting = 0;
	mat.color = 0x446666ff;

	var sh = Shader.new(1 + 2);
	sh.vertex = """
		uniform mat4 u_projmodmat,u_modmat,u_viewmatinv;
		uniform mat3 u_rotmat;
		uniform vec3 u_campos;
		attribute vec4 a_vert;
		attribute vec3 a_norm;
		varying mediump vec3 v_wnorm, v_lnorm, v_dir;
		void main()
		{
			gl_Position = u_projmodmat * a_vert;
			v_lnorm = normalize(u_rotmat * a_norm);
			v_wnorm = normalize(mat3(u_viewmatinv) * v_lnorm);
			vec3 pos = (u_viewmatinv * (u_modmat * a_vert)).xyz;
			v_dir = u_campos - pos;
		}
	""";
	sh.fragment = """
		uniform sampler2D u_tex;
		varying mediump vec3 v_lnorm, v_wnorm, v_dir;
		void main()
		{
			mediump vec3 refl = normalize(reflect(-v_dir, v_wnorm));
			mediump vec2 index;
			index.x = atan(-refl.x, -refl.y) / 6.283;
			index.y = clamp((-asin(refl.z) / 3.15) + 0.5, 0.1, 0.9);
			lowp vec4 t = texture2D(u_tex, index);
			t.a = max(pow(1.0 - v_lnorm.z, 0.5), t.a);
			gl_FragColor = t;
		}
	""";
	mat.texture = "whiteworld_env";
	mat.shader = sh;

	return mat;
}

function addBottleObject()
{
	var o = Obj3D.newEmpty();
	o.add(Obj3D.load("demoworld")).transform = Matrix.newTranslation((0, 0, -0.4));
	o.add(Obj3D.load("coverlabel")).transform = Matrix.newTranslation((0, 0, -0.4));
	var cover;
	cover = Obj3D.load("coverhole");
	o.add(cover).transform = Matrix.newTranslation((0, 0, -0.4));
	cover.transparent = 1;
	cover.material = createGlassMaterial();

	GLDisplay.world3d.add(o);
	return o;
}
